//-------------------------------------------STM32F4_DMA_PWM-------------------------------------------------------

/* Beschreibung:
 * Beispiel wie per DMA praktisch beliebig viele PWM-Signale auf beliebigen Ausgabepins
 * erzeugt werden knnen ohne Interrupt und ohne CPU-Belastung.
 */
/*

Hinweis zum Clock fuer 8MHz Quarz, damit er auf 168MHz luft (betrifft das stm32F4Discovery):

in der system_stm32f4xx.c muss eingetragen sein:
PLL_M = 8

in der stm32f4xx.h muss eingetragen sein:
HSE_VALUE = 8000000
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "stm32f4xx.h"
#include "stm32f4xx_gpio.h"
#include "stm32f4xx_rcc.h"
#include "stm32f4xx_tim.h"
#include "misc.h"
#include "stm32f4xx_dma.h"

// Adresse des ODR-Registers des PORTS auf dem die PWM-Signale ausgegeben werden (in diesem Fall PORTA)
#define GPIOA_ODR_ADRESS	   0x40020014

// Anzahl der Schritte mit der die PWM eingestellt werden kann (Auflsung)
#define PWM_RES 100

// Der Speicher in dem die PWM-Signale abgebildet werden
uint16_t Pwm_Memory[PWM_RES] = {0};

// Der Timer, der den DMA-Transfer auslst
void TIM1_Config(void)
{
  TIM_TimeBaseInitTypeDef    TIM_TimeBaseStructure;

  RCC_APB2PeriphClockCmd(RCC_APB2Periph_TIM1, ENABLE);

  /* Time base configuration */
  TIM_TimeBaseStructInit(&TIM_TimeBaseStructure);
  TIM_TimeBaseStructure.TIM_Period = 0xF;
  TIM_TimeBaseStructure.TIM_Prescaler = 0;
  TIM_TimeBaseStructure.TIM_ClockDivision = TIM_CKD_DIV1;
  TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;
  TIM_TimeBaseInit(TIM1, &TIM_TimeBaseStructure);

  /* TIM6 enable counter */
  TIM_Cmd(TIM1, ENABLE);
}

// Die DMA und der GPIO-PORT wird initialisiert
void DMA_PWM_INIT(void)
{

  GPIO_InitTypeDef GPIO_InitStructure;
  DMA_InitTypeDef DMA_InitStructure;

  RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_DMA2 | RCC_AHB1Periph_GPIOA, ENABLE);

  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0 | GPIO_Pin_1 | GPIO_Pin_2 | GPIO_Pin_3 | GPIO_Pin_4 | GPIO_Pin_5; // ... einfach erweitern, wenn mehr PWM-Kanle bentigt werden
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_100MHz;
  GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
  GPIO_Init(GPIOA, &GPIO_InitStructure);

//TIM1 Configuration
  TIM1_Config();

    DMA_DeInit(DMA2_Stream5); // welcher Stream/Channel verwendet wird ergibt sich aus der DMA2 request mapping-Tabelle im Reference-Datasheet auf S.165
    DMA_InitStructure.DMA_Channel = DMA_Channel_6;
    DMA_InitStructure.DMA_PeripheralBaseAddr = (uint32_t)GPIOA_ODR_ADRESS;
    DMA_InitStructure.DMA_Memory0BaseAddr = (uint32_t)&Pwm_Memory;
    DMA_InitStructure.DMA_DIR = DMA_DIR_MemoryToPeripheral;
    DMA_InitStructure.DMA_BufferSize = PWM_RES; // Gibt an wie lang die Sequenz sein soll, die dann auf dem PORT ausgegeben wird.
    DMA_InitStructure.DMA_PeripheralInc = DMA_PeripheralInc_Disable;
    DMA_InitStructure.DMA_MemoryInc = DMA_MemoryInc_Enable;
    DMA_InitStructure.DMA_PeripheralDataSize = DMA_PeripheralDataSize_HalfWord;
    DMA_InitStructure.DMA_MemoryDataSize = DMA_MemoryDataSize_HalfWord;
    DMA_InitStructure.DMA_Mode = DMA_Mode_Circular;
    DMA_InitStructure.DMA_Priority = DMA_Priority_High;
    DMA_InitStructure.DMA_FIFOMode = DMA_FIFOMode_Disable;
    DMA_InitStructure.DMA_FIFOThreshold = DMA_FIFOThreshold_HalfFull;
    DMA_InitStructure.DMA_MemoryBurst = DMA_MemoryBurst_Single;
    DMA_InitStructure.DMA_PeripheralBurst = DMA_PeripheralBurst_Single;
    DMA_Init(DMA2_Stream5, &DMA_InitStructure);

    DMA_Cmd(DMA2_Stream5, ENABLE);

    TIM_DMACmd(TIM1,TIM_DMA_Update,ENABLE);
}

// diese Funktion setzt den Speicher in dem die PWM-Signale abgebildet werden
// Das Argument "pwm_werte" muss ein Array aus 16 Werten sein.
void DMA_PWM_SET(uint16_t *pwm_werte)
{
	uint16_t n,x;

	for (n = 0; n < PWM_RES; n++)
	{
		for (x = 0; x < 16;x++)
		{
			if (pwm_werte[x] > n)
			{
				Pwm_Memory[n] |= (1<<x);
			}
			else
			{
				Pwm_Memory[n] &= ~(1<<x);
			}
		}
	}
}


int main(void)
{
	uint16_t pwm_kanalwert[16] = {0};

	// Die PLL wird auf 168 MHz eingestellt
	SystemInit();

	// Die DMA und der GPIO-PORT wird initialisiert
	DMA_PWM_INIT();

	// Einige Kanalwerte setzen:
	pwm_kanalwert[0] = 1;
	pwm_kanalwert[1] = 10;
	pwm_kanalwert[2] = 20;
	pwm_kanalwert[3] = 30;
	pwm_kanalwert[4] = 40;
	pwm_kanalwert[5] = 50;
	//... einfach erweitern, wenn mehr PWM-Kanle gesetzt werden sollen

	// Nun werden die Kanalwerte in den Speicher bernommen, auf welchen wiederrum die DMA dann zugreift.
	DMA_PWM_SET(pwm_kanalwert);

	while(1)
    {

    }
}
